"use client";

import { useState, createContext, useContext, useEffect, ReactNode } from 'react';
import Cookies from 'js-cookie';


// Define the type for the context
interface AuthContextType {
  token: string | null;
  isTokenLoaded: boolean;
  userInfo: any; // Replace 'any' with the actual type of userInfo
  login: (token: string, persist: boolean, exp: number) => void;
  logout: () => void;
  setUserInfo: React.Dispatch<React.SetStateAction<any>>; // Replace 'any' with the actual type of userInfo
  navTitle: string,
  setNavTitle: React.Dispatch<React.SetStateAction<string>>;
 }
 const initialState: AuthContextType = {
  token: null,
  isTokenLoaded: false,
  userInfo: null,
  login: () => {},
  logout: () => {},
  setUserInfo: () => {},
  navTitle: 'Dashboard',
  setNavTitle: () =>{}
 }
 
 // Create the context with a default value
 export const AuthContext = createContext<AuthContextType>(initialState);
 
 // Custom hook to use the context
 export const useAuth = (): AuthContextType => {
  const context = useContext(AuthContext);
  if (!context) {
     throw new Error('useAuth must be used within an AuthProvider');
  }
  return context;
};

interface AuthProviderProps {
  children: ReactNode;
 }
 
 export const AuthProvider = ({ children }: AuthProviderProps) => {
  const [token, setToken] = useState<string | null>(null);
  const [isTokenLoaded, setIsTokenLoaded] = useState<boolean>(false);
  const [userInfo, setUserInfo] = useState<any>(null); // Replace 'any' with the actual type of userInfo
  const [navTitle, setNavTitle] = useState<string>('Dashboard');
 
  useEffect(() => {
     const fetchToken = async () => {
       const storedToken = Cookies.get('_token');
       if (storedToken) {
         setToken(storedToken);
       }
       setIsTokenLoaded(true);
     };
 
     fetchToken();
  }, []);
 
  useEffect(() => {
     if (!isTokenLoaded) return;
 
     if (!token) {
       // Handle the case where there is no token
     } else {
       // Proceed with your logic that depends on the token
     }
  }, [token, isTokenLoaded]);
 
  const login = (token: string, persist: boolean, exp: number | null) => {
    let day = 1;
    if(exp !== null){
      const currentTime = Math.floor(Date.now() / 1000);
      const durationInSeconds = exp - currentTime;

      // Convert duration from seconds to days
      const durationInDays = durationInSeconds / (60 * 60 * 24);
      day = durationInDays;
    }

     setToken(token);
     Cookies.set('_token', token, {
       expires: day, // 7 Days
       path: '',
     });
  };
 
  const logout = () => {
     setToken(null);
     Cookies.remove('_token');
  };
 
  return (
    <AuthContext.Provider value={{ token, login, logout, isTokenLoaded, userInfo, setUserInfo, navTitle, setNavTitle }}>
      {children}
    </AuthContext.Provider>
  );

 }